@@ -1511,8 +1511,7 @@ namespace fheroes2
                 temp = getBoundaryRect( temp, cursorROI );
             }
 
-            // Previous position of cursor must be updated as well to avoid ghost effect.
-            _renderFrame( getBoundaryRect( temp, _prevRoi ) );
+            _renderFrame( temp );
 
             if ( _postprocessing ) {
                 _postprocessing();
@@ -1521,7 +1520,7 @@ namespace fheroes2
             Copy( backup, 0, 0, *this, backup.x(), backup.y(), backup.width(), backup.height() );
         }
         else {
-            _renderFrame( getBoundaryRect( temp, _prevRoi ) );
+            _renderFrame( temp );
 
             if ( _postprocessing ) {
                 _postprocessing();
@@ -1555,7 +1554,8 @@ namespace fheroes2
         }
 
         if ( updateImage ) {
-            _engine->render( *this, roi );
+            // Make sure that we update the previously rendered area to avoid any ghost effect artefacts.
+            _engine->render( *this, getBoundaryRect( roi, _prevRoi ) );
         }
     }
 
