@@ -25,7 +25,6 @@
 #include <cassert>
 #include <cstddef>
 #include <cstdint>
-#include <initializer_list>
 #include <map>
 #include <memory>
 #include <ostream>
@@ -139,26 +138,6 @@ namespace
         // Index of the destination cell should correspond to the index of the head cell of the target position and nothing else
         return pos.GetHead()->GetIndex() == dst;
     }
-
-    std::pair<uint32_t, uint32_t> getEarthquakeDamageRange( const HeroBase * commander )
-    {
-        const int spellPower = commander->GetPower();
-        if ( ( spellPower > 0 ) && ( spellPower < 3 ) ) {
-            return { 0, 1 };
-        }
-        if ( ( spellPower >= 3 ) && ( spellPower < 6 ) ) {
-            return { 0, 2 };
-        }
-        if ( ( spellPower >= 6 ) && ( spellPower < 10 ) ) {
-            return { 0, 3 };
-        }
-        if ( spellPower >= 10 ) {
-            return { 1, 3 };
-        }
-
-        DEBUG_LOG( DBG_BATTLE, DBG_WARN, "unexpected spellPower value: " << spellPower << " for commander " << commander )
-        return { 0, 0 };
-    }
 }
 
 void Battle::Arena::BattleProcess( Unit & attacker, Unit & defender, int32_t tgt /* = -1 */, int dir /* = -1 */ )
@@ -1550,9 +1529,7 @@ void Battle::Arena::_applyActionSpellEarthquake()
 
     std::map<CastleDefenseStructure, int> earthquakeDamage;
 
-    for ( const CastleDefenseStructure target :
-          { CastleDefenseStructure::WALL1, CastleDefenseStructure::WALL2, CastleDefenseStructure::WALL3, CastleDefenseStructure::WALL4, CastleDefenseStructure::TOWER1,
-            CastleDefenseStructure::TOWER2, CastleDefenseStructure::BRIDGE, CastleDefenseStructure::TOP_BRIDGE_TOWER, CastleDefenseStructure::BOTTOM_BRIDGE_TOWER } ) {
+    for ( const CastleDefenseStructure target : getEarthQuakeSpellTargets() ) {
         const int targetCondition = getCastleDefenseStructureCondition( target, SiegeWeaponType::EarthquakeSpell );
         assert( targetCondition >= 0 );
 
