@@ -221,11 +221,19 @@ namespace Interface
     fheroes2::Rect EditorPanel::getBrushArea() const
     {
         // Roads and streams are placed using only 1x1 brush.
-        if ( _selectedInstrument == Instrument::STREAM || _selectedInstrument == Instrument::ROAD || _selectedInstrument == Instrument::DETAIL
-             || _selectedInstrument == Instrument::MONSTERS ) {
+        if ( _selectedInstrument == Instrument::STREAM || _selectedInstrument == Instrument::ROAD || _selectedInstrument == Instrument::DETAIL ) {
             return { 0, 0, 1, 1 };
         }
 
+        if ( _selectedInstrument == Instrument::MONSTERS ) {
+            const int32_t objectType = getSelectedObjectType();
+            if ( objectType >= 0 ) {
+                return { 0, 0, 1, 1 };
+            }
+
+            return {};
+        }
+
         if ( _selectedInstrument == Instrument::LANDSCAPE_OBJECTS || _selectedInstrument == Instrument::ADVENTURE_OBJECTS
              || _selectedInstrument == Instrument::KINGDOM_OBJECTS ) {
             const int32_t objectType = getSelectedObjectType();
