@@ -96,7 +96,7 @@ namespace Interface
 
         bool useMouseDragMovement() const
         {
-            return !( ( _selectedInstrument == Instrument::TERRAIN || _selectedInstrument == Instrument::ERASE ) && _selectedBrushSize == BrushSize::AREA );
+            return ( _selectedInstrument != Instrument::TERRAIN && _selectedInstrument != Instrument::ERASE );
         }
 
         // Set Editor panel positions on screen.
@@ -116,7 +116,6 @@ namespace Interface
         Maps::ObjectGroup getSelectedObjectGroup() const;
 
         void getTownObjectProperties( int32_t & type, int32_t & color ) const;
-        void getMineObjectProperties( int32_t & type, int32_t & color ) const;
 
         static const char * getObjectGroupName( const Maps::ObjectGroup groupName );
 
@@ -136,7 +135,6 @@ namespace Interface
         static const char * _getEraseObjectTypeName( const uint32_t eraseObjectType );
 
         static int32_t _generateTownObjectProperties( const int32_t type, const int32_t color );
-        static int32_t _generateMineObjectProperties( const int32_t type, const int32_t color );
 
         void _setCursor();
 
