@@ -117,7 +117,7 @@ namespace
         Game::setDisplayFadeIn();
 
         Interface::EditorInterface & editorInterface = Interface::EditorInterface::Get();
-        if ( !editorInterface.updateRandomMapConfiguration() ) {
+        if ( !editorInterface.updateRandomMapConfiguration( mapSize ) ) {
             return fheroes2::GameMode::EDITOR_NEW_MAP;
         }
 
@@ -290,11 +290,6 @@ namespace Editor
                     generateRandomMap = true;
 
                     prepareMapSizeMenu();
-
-                    fheroes2::showStandardTextMessage(
-                        _( "Warning" ),
-                        "This feature is still in development and has some limitations. Errors might occur. This feature will continue to change as we are working on improving it.",
-                        Dialog::OK );
                 }
                 else if ( le.MouseClickLeft( buttonBack.area() ) || Game::HotKeyPressEvent( Game::HotKeyEvent::DEFAULT_CANCEL ) ) {
                     mapCreationModeButtons.disable();
