@@ -411,16 +411,9 @@ namespace
     }
 }
 
-void Maps::Tile::Init( int32_t index, const MP2::MP2TileInfo & mp2 )
+void Maps::Tile::Init( const MP2::MP2TileInfo & mp2 )
 {
-    _tilePassabilityDirections = DIRECTION_ALL;
-
     _metadata[0] = ( ( ( mp2.quantity2 << 8 ) + mp2.quantity1 ) >> 3 );
-    _fogColors = Color::ALL;
-    _terrainImageIndex = mp2.terrainImageIndex;
-    _terrainFlags = mp2.terrainFlags;
-    _boatOwnerColor = Color::NONE;
-    _index = index;
 
     setMainObjectType( static_cast<MP2::MapObjectType>( mp2.mapObjectType ) );
 
@@ -431,59 +424,33 @@ void Maps::Tile::Init( int32_t index, const MP2::MP2TileInfo & mp2 )
                    "Metadata present for non action object " << MP2::StringObject( _mainObjectType ) << " at tile " << _index << ". Metadata value " << _metadata[0] )
     }
 
-    _groundObjectPart.clear();
-    _topObjectPart.clear();
-
-    const MP2::ObjectIcnType bottomObjectIcnType = static_cast<MP2::ObjectIcnType>( mp2.objectName1 >> 2 );
-
-    const ObjectLayerType layerType = static_cast<ObjectLayerType>( mp2.quantity1 & 0x03 );
-
     // In the original Editor the road bit is set even if no road exist.
     // It is important to verify the existence of a road without relying on this bit.
-    if ( isSpriteRoad( bottomObjectIcnType, mp2.bottomIcnImageIndex ) ) {
-        _isTileMarkedAsRoad = true;
-    }
 
-    if ( _mainObjectType == MP2::OBJ_NONE && ( layerType == ObjectLayerType::SHADOW_LAYER || layerType == ObjectLayerType::TERRAIN_LAYER ) ) {
-        // If an object sits on shadow or terrain layer then we should put it as a bottom layer add-on.
-        if ( bottomObjectIcnType != MP2::ObjectIcnType::OBJ_ICN_TYPE_UNKNOWN ) {
-            _groundObjectPart.emplace_back( layerType, mp2.level1ObjectUID, bottomObjectIcnType, mp2.bottomIcnImageIndex );
-        }
-    }
-    else {
-        _mainObjectPart.layerType = layerType;
-        _mainObjectPart._uid = mp2.level1ObjectUID;
-        _mainObjectPart.icnType = bottomObjectIcnType;
-        _mainObjectPart.icnIndex = mp2.bottomIcnImageIndex;
+    const MP2::ObjectIcnType bottomObjectIcnType = static_cast<MP2::ObjectIcnType>( mp2.objectName1 >> 2 );
+    if ( bottomObjectIcnType != MP2::ObjectIcnType::OBJ_ICN_TYPE_UNKNOWN ) {
+        pushGroundObjectPart( { static_cast<ObjectLayerType>( mp2.quantity1 & 0x03 ), mp2.level1ObjectUID, bottomObjectIcnType, mp2.bottomIcnImageIndex } );
     }
 
     const MP2::ObjectIcnType topObjectIcnType = static_cast<MP2::ObjectIcnType>( mp2.objectName2 >> 2 );
     if ( topObjectIcnType != MP2::ObjectIcnType::OBJ_ICN_TYPE_UNKNOWN ) {
         // Top layer objects do not have any internal structure (layers) so all of them should have the same internal layer.
         // TODO: remove layer type for top layer objects.
-        _topObjectPart.emplace_back( OBJECT_LAYER, mp2.level2ObjectUID, topObjectIcnType, mp2.topIcnImageIndex );
+
+        // One object part for a crack object has invalid layer.
+        // This causes a rendering bug which is also present in the original game.
+        if ( topObjectIcnType == MP2::OBJ_ICN_TYPE_OBJNCRCK && mp2.topIcnImageIndex == 226 ) {
+            pushGroundObjectPart( { ObjectLayerType::TERRAIN_LAYER, mp2.level2ObjectUID, topObjectIcnType, mp2.topIcnImageIndex } );
+        }
+        else {
+            pushTopObjectPart( { OBJECT_LAYER, mp2.level2ObjectUID, topObjectIcnType, mp2.topIcnImageIndex } );
+        }
     }
 }
 
-void Maps::Tile::setTerrain( const uint16_t terrainImageIndex, const bool horizontalFlip, const bool verticalFlip )
-{
-    _terrainFlags = ( verticalFlip ? 1 : 0 ) + ( horizontalFlip ? 2 : 0 );
-
-    const int newGround = Ground::getGroundByImageIndex( terrainImageIndex );
-
-    if ( ( _isTileMarkedAsRoad || isStream() ) && ( newGround != Ground::WATER ) && Ground::doesTerrainImageIndexContainEmbeddedObjects( terrainImageIndex ) ) {
-        // There cannot be extra objects under the roads and streams.
-        _terrainImageIndex = Ground::getRandomTerrainImageIndex( Ground::getGroundByImageIndex( terrainImageIndex ), false );
-
-        return;
-    }
-
-    _terrainImageIndex = terrainImageIndex;
-}
-
 Heroes * Maps::Tile::getHero() const
 {
-    return MP2::OBJ_HERO == _mainObjectType && Heroes::isValidId( _occupantHeroId ) ? world.GetHeroes( _occupantHeroId ) : nullptr;
+    return ( MP2::OBJ_HERO == _mainObjectType ) && Heroes::isValidId( _occupantHeroId ) ? world.GetHeroes( _occupantHeroId ) : nullptr;
 }
 
 void Maps::Tile::setHero( Heroes * hero )
@@ -519,14 +486,14 @@ fheroes2::Point Maps::Tile::GetCenter() const
     return GetPoint( _index );
 }
 
-MP2::MapObjectType Maps::Tile::getMainObjectType( const bool ignoreObjectUnderHero /* true */ ) const
+MP2::MapObjectType Maps::Tile::_getMainObjectTypeUnderHero() const
 {
-    if ( !ignoreObjectUnderHero && MP2::OBJ_HERO == _mainObjectType ) {
-        const Heroes * hero = getHero();
-        return hero ? hero->getObjectTypeUnderHero() : MP2::OBJ_NONE;
+    if ( _mainObjectType != MP2::OBJ_HERO ) {
+        return _mainObjectType;
     }
 
-    return _mainObjectType;
+    const Heroes * hero = getHero();
+    return hero ? hero->getObjectTypeUnderHero() : MP2::OBJ_NONE;
 }
 
 void Maps::Tile::setMainObjectType( const MP2::MapObjectType objectType )
@@ -536,7 +503,7 @@ void Maps::Tile::setMainObjectType( const MP2::MapObjectType objectType )
     world.resetPathfinder();
 }
 
-void Maps::Tile::setBoat( const int direction, const int color )
+void Maps::Tile::setBoat( const int direction, const PlayerColor color )
 {
     if ( _mainObjectPart.icnType != MP2::OBJ_ICN_TYPE_UNKNOWN ) {
         // It is important to preserve the order of objects for rendering purposes. Therefore, the main object should go to the front of objects.
@@ -547,6 +514,12 @@ void Maps::Tile::setBoat( const int direction, const int color )
     assert( isWater() );
 
     setMainObjectType( MP2::OBJ_BOAT );
+
+    // Make sure that the object part is being cleared before setting a new one.
+    _mainObjectPart = {};
+
+    // A boat is an object so it must be on the object layer.
+    _mainObjectPart.layerType = ObjectLayerType::OBJECT_LAYER;
     _mainObjectPart.icnType = MP2::OBJ_ICN_TYPE_BOAT32;
 
     switch ( direction ) {
@@ -580,6 +553,9 @@ void Maps::Tile::setBoat( const int direction, const int color )
         break;
     }
 
+    // Boats are not passable.
+    assert( !_mainObjectPart.isPassabilityTransparent() );
+
 #ifdef WITH_DEBUG
     const uint32_t newUid = getNewObjectUID();
 
@@ -592,12 +568,7 @@ void Maps::Tile::setBoat( const int direction, const int color )
     _mainObjectPart._uid = getNewObjectUID();
 #endif // WITH_DEBUG
 
-    using BoatOwnerColorType = decltype( _boatOwnerColor );
-    static_assert( std::is_same_v<BoatOwnerColorType, uint8_t> );
-
-    assert( color >= std::numeric_limits<BoatOwnerColorType>::min() && color <= std::numeric_limits<BoatOwnerColorType>::max() );
-
-    _boatOwnerColor = static_cast<BoatOwnerColorType>( color );
+    _boatOwnerColor = color;
 }
 
 int Maps::Tile::getBoatDirection() const
@@ -648,8 +619,8 @@ int Maps::Tile::getTileIndependentPassability() const
     int passability = DIRECTION_ALL;
 
     const auto getObjectPartPassability = []( const Maps::ObjectPart & part, bool & isActionObject ) {
-        if ( part.icnType == MP2::OBJ_ICN_TYPE_ROAD || part.icnType == MP2::OBJ_ICN_TYPE_STREAM ) {
-            // Rivers and stream are completely passable.
+        if ( part.icnType == MP2::OBJ_ICN_TYPE_ROAD || part.icnType == MP2::OBJ_ICN_TYPE_STREAM || part.icnType == MP2::OBJ_ICN_TYPE_FLAG32 ) {
+            // Rivers, streams and flags are completely passable.
             return DIRECTION_ALL;
         }
 
@@ -801,13 +772,13 @@ void Maps::Tile::updatePassability()
         return;
     }
 
-    // Count how many objects are there excluding shadows, roads and river streams.
+    // Count how many objects are there excluding shadows, roads, river streams and flags.
     const std::ptrdiff_t validBottomLayerObjects = std::count_if( _groundObjectPart.begin(), _groundObjectPart.end(), []( const auto & part ) {
         if ( isObjectPartShadow( part ) ) {
             return false;
         }
 
-        return part.icnType != MP2::OBJ_ICN_TYPE_ROAD && part.icnType != MP2::OBJ_ICN_TYPE_STREAM;
+        return part.icnType != MP2::OBJ_ICN_TYPE_ROAD && part.icnType != MP2::OBJ_ICN_TYPE_STREAM && part.icnType != MP2::OBJ_ICN_TYPE_FLAG32;
     } );
 
     const bool singleObjectTile = ( validBottomLayerObjects == 0 ) && _topObjectPart.empty() && ( bottomTile._mainObjectPart.icnType != _mainObjectPart.icnType );
@@ -849,53 +820,13 @@ bool Maps::Tile::doesObjectExist( const uint32_t uid ) const
                         [uid]( const auto & part ) { return part._uid == uid && !part.isPassabilityTransparent(); } );
 }
 
-void Maps::Tile::UpdateRegion( uint32_t newRegionID )
+void Maps::Tile::pushGroundObjectPart( ObjectPart part )
 {
-    if ( _tilePassabilityDirections ) {
-        _region = newRegionID;
-    }
-    else {
-        _region = REGION_NODE_BLOCKED;
-    }
-}
-
-void Maps::Tile::pushGroundObjectPart( const MP2::MP2AddonInfo & ma )
-{
-    const MP2::ObjectIcnType objectIcnType = static_cast<MP2::ObjectIcnType>( ma.objectNameN1 >> 2 );
-    if ( objectIcnType == MP2::ObjectIcnType::OBJ_ICN_TYPE_UNKNOWN ) {
-        // No object exist.
-        return;
-    }
-
-    // In the original Editor the road bit is set even if no road exist.
-    // It is important to verify the existence of a road without relying on this bit.
-    if ( isSpriteRoad( objectIcnType, ma.bottomIcnImageIndex ) ) {
+    if ( isSpriteRoad( part.icnType, part.icnIndex ) ) {
         _isTileMarkedAsRoad = true;
     }
 
-    _groundObjectPart.emplace_back( static_cast<ObjectLayerType>( ma.quantityN & 0x03 ), ma.level1ObjectUID, objectIcnType, ma.bottomIcnImageIndex );
-}
-
-void Maps::Tile::pushTopObjectPart( const MP2::MP2AddonInfo & ma )
-{
-    const MP2::ObjectIcnType objectIcnType = static_cast<MP2::ObjectIcnType>( ma.objectNameN2 >> 2 );
-    if ( objectIcnType == MP2::ObjectIcnType::OBJ_ICN_TYPE_UNKNOWN ) {
-        // No object exist.
-        return;
-    }
-
-    // Top layer objects do not have any internal structure (layers) so all of them should have the same internal layer.
-    // TODO: remove layer type for top layer objects.
-    _topObjectPart.emplace_back( OBJECT_LAYER, ma.level2ObjectUID, objectIcnType, ma.topIcnImageIndex );
-}
-
-void Maps::Tile::pushGroundObjectPart( ObjectPart ta )
-{
-    if ( isSpriteRoad( ta.icnType, ta.icnIndex ) ) {
-        _isTileMarkedAsRoad = true;
-    }
-
-    _groundObjectPart.emplace_back( ta );
+    _groundObjectPart.emplace_back( part );
 }
 
 void Maps::Tile::sortObjectParts()
@@ -914,13 +845,23 @@ void Maps::Tile::sortObjectParts()
     _groundObjectPart.sort( []( const auto & left, const auto & right ) { return ( left.layerType > right.layerType ); } );
 
     if ( !_groundObjectPart.empty() ) {
-        ObjectPart & highestPriorityPart = _groundObjectPart.back();
-        std::swap( highestPriorityPart, _mainObjectPart );
+        auto highestPriorityPartIter = _groundObjectPart.end();
 
-        // If this assertion blows up then you are not storing correct values for layer type!
-        assert( _mainObjectPart.layerType <= TERRAIN_LAYER );
+        while ( highestPriorityPartIter != _groundObjectPart.begin() ) {
+            --highestPriorityPartIter;
 
-        _groundObjectPart.pop_back();
+            // Flags might be on the same layer as other objects but they cannot be the main object because they are a part of another object.
+            if ( highestPriorityPartIter->icnType != MP2::OBJ_ICN_TYPE_FLAG32 ) {
+                std::swap( *highestPriorityPartIter, _mainObjectPart );
+
+                // If this assertion blows up then you are not storing correct values for layer type!
+                assert( _mainObjectPart.layerType <= TERRAIN_LAYER );
+
+                _groundObjectPart.erase( highestPriorityPartIter );
+
+                break;
+            }
+        }
     }
 
     // Top layer objects don't have any rendering priorities so they should be rendered first in queue first to render.
@@ -1062,9 +1003,9 @@ std::string Maps::Tile::String() const
     return os.str();
 }
 
-bool Maps::Tile::GoodForUltimateArtifact() const
+bool Maps::Tile::isSuitableForUltimateArtifact() const
 {
-    if ( isWater() || !isPassableFrom( Direction::CENTER, false, true, 0 ) ) {
+    if ( isWater() || !isPassableFrom( Direction::CENTER, false, true, PlayerColor::NONE ) ) {
         return false;
     }
 
@@ -1083,6 +1024,69 @@ bool Maps::Tile::GoodForUltimateArtifact() const
     return true;
 }
 
+bool Maps::Tile::isSuitableForDisembarkation() const
+{
+    // Tiles with OBJ_COAST are always suitable for disembarkation
+    if ( _mainObjectType == MP2::OBJ_COAST ) {
+        return true;
+    }
+
+    // Tiles with heroes are a special case because heroes are moving objects and, strictly speaking, are not part of the map itself, so the checks below do not work with
+    // them
+    if ( _mainObjectType == MP2::OBJ_HERO ) {
+        return false;
+    }
+
+    // Tiles with events are not suitable for disembarkation (at least for now)
+    if ( _mainObjectType == MP2::OBJ_EVENT ) {
+        return false;
+    }
+
+    // It is impossible to disembark on water tiles
+    if ( isWater() ) {
+        return false;
+    }
+
+    const auto isObjectPartPassable = []( const Maps::ObjectPart & part ) {
+        // Roads, streams and flags are completely passable.
+        if ( part.icnType == MP2::OBJ_ICN_TYPE_ROAD || part.icnType == MP2::OBJ_ICN_TYPE_STREAM || part.icnType == MP2::OBJ_ICN_TYPE_FLAG32 ) {
+            return true;
+        }
+
+        return part.isPassabilityTransparent() || isObjectPartShadow( part );
+    };
+
+    return ( _mainObjectPart.icnType == MP2::OBJ_ICN_TYPE_UNKNOWN || isObjectPartPassable( _mainObjectPart ) )
+           && std::all_of( _groundObjectPart.begin(), _groundObjectPart.end(), isObjectPartPassable );
+}
+
+bool Maps::Tile::isSuitableForSummoningBoat() const
+{
+    // Tiles with heroes are a special case because heroes are moving objects and, strictly speaking, are not part of the map itself, so the checks below do not work with
+    // them
+    if ( _mainObjectType == MP2::OBJ_HERO ) {
+        return false;
+    }
+
+    // Tiles with events are not suitable for summoning a boat (at least for now)
+    if ( _mainObjectType == MP2::OBJ_EVENT ) {
+        return false;
+    }
+
+    // It is impossible to summon a boat on land
+    if ( !isWater() ) {
+        return false;
+    }
+
+    const auto isObjectPartPassable = []( const Maps::ObjectPart & part ) {
+        // The part of the Lighthouse flag that is located on the adjacent tile doesn't affect the passability
+        return part.icnType == MP2::OBJ_ICN_TYPE_FLAG32 || part.isPassabilityTransparent() || isObjectPartShadow( part );
+    };
+
+    return ( _mainObjectPart.icnType == MP2::OBJ_ICN_TYPE_UNKNOWN || isObjectPartPassable( _mainObjectPart ) )
+           && std::all_of( _groundObjectPart.begin(), _groundObjectPart.end(), isObjectPartPassable );
+}
+
 bool Maps::Tile::isPassabilityTransparent() const
 {
     for ( const auto & part : _groundObjectPart ) {
@@ -1094,7 +1098,7 @@ bool Maps::Tile::isPassabilityTransparent() const
     return _mainObjectPart.isPassabilityTransparent();
 }
 
-bool Maps::Tile::isPassableFrom( const int direction, const bool fromWater, const bool ignoreFog, const int heroColor ) const
+bool Maps::Tile::isPassableFrom( const int direction, const bool fromWater, const bool ignoreFog, const PlayerColor heroColor ) const
 {
     if ( !ignoreFog && isFog( heroColor ) ) {
         return false;
@@ -1102,13 +1106,19 @@ bool Maps::Tile::isPassableFrom( const int direction, const bool fromWater, cons
 
     const bool tileIsWater = isWater();
 
-    // From the water we can get either to the coast tile or to the water tile (provided there is no boat on this tile).
-    if ( fromWater && _mainObjectType != MP2::OBJ_COAST && ( !tileIsWater || _mainObjectType == MP2::OBJ_BOAT ) ) {
-        return false;
+    // From the water we can get either to a water tile (provided that there is no boat on that tile) or to a shore tile suitable for disembarkation.
+    if ( fromWater ) {
+        if ( tileIsWater ) {
+            if ( _mainObjectType == MP2::OBJ_BOAT ) {
+                return false;
+            }
+        }
+        else if ( !isSuitableForDisembarkation() ) {
+            return false;
+        }
     }
-
-    // From the ground we can get to the water tile only if this tile contains a certain object.
-    if ( !fromWater && tileIsWater && _mainObjectType != MP2::OBJ_SHIPWRECK && _mainObjectType != MP2::OBJ_HERO && _mainObjectType != MP2::OBJ_BOAT ) {
+    // From the ground we can get to a water tile only if this tile contains a certain object.
+    else if ( tileIsWater && _mainObjectType != MP2::OBJ_SHIPWRECK && _mainObjectType != MP2::OBJ_HERO && _mainObjectType != MP2::OBJ_BOAT ) {
         return false;
     }
 
@@ -1171,35 +1181,35 @@ Maps::ObjectPart * Maps::Tile::getObjectPartWithFlag( const uint32_t uid )
     return nullptr;
 }
 
-void Maps::Tile::setOwnershipFlag( const MP2::MapObjectType objectType, int color )
+void Maps::Tile::setOwnershipFlag( const MP2::MapObjectType objectType, PlayerColor color )
 {
     // All flags in FLAG32.ICN are actually the same except the fact of having different offset.
     // Set the default value for the UNUSED color.
     uint8_t objectSpriteIndex = 6;
 
     switch ( color ) {
-    case Color::NONE:
+    case PlayerColor::NONE:
         // No flag. Just ignore it.
         break;
-    case Color::BLUE:
+    case PlayerColor::BLUE:
         objectSpriteIndex = 0;
         break;
-    case Color::GREEN:
+    case PlayerColor::GREEN:
         objectSpriteIndex = 1;
         break;
-    case Color::RED:
+    case PlayerColor::RED:
         objectSpriteIndex = 2;
         break;
-    case Color::YELLOW:
+    case PlayerColor::YELLOW:
         objectSpriteIndex = 3;
         break;
-    case Color::ORANGE:
+    case PlayerColor::ORANGE:
         objectSpriteIndex = 4;
         break;
-    case Color::PURPLE:
+    case PlayerColor::PURPLE:
         objectSpriteIndex = 5;
         break;
-    case Color::UNUSED:
+    case PlayerColor::UNUSED:
         // Should never be called using this color as an argument.
         assert( 0 );
         break;
@@ -1246,8 +1256,8 @@ void Maps::Tile::setOwnershipFlag( const MP2::MapObjectType objectType, int colo
 
     case MP2::OBJ_CASTLE:
         // Neutral castles always have flags on both sides of the gate, they should not be completely removed.
-        if ( color == Color::NONE ) {
-            color = Color::UNUSED;
+        if ( color == PlayerColor::NONE ) {
+            color = PlayerColor::UNUSED;
         }
 
         objectSpriteIndex *= 2;
@@ -1268,10 +1278,10 @@ void Maps::Tile::setOwnershipFlag( const MP2::MapObjectType objectType, int colo
     }
 }
 
-void Maps::Tile::updateFlag( const int color, const uint8_t objectSpriteIndex, const uint32_t uid, const bool setOnUpperLayer )
+void Maps::Tile::updateFlag( const PlayerColor color, const uint8_t objectSpriteIndex, const uint32_t uid, const bool setOnUpperLayer )
 {
     // Flag deletion or installation must be done in relation to object UID as flag is attached to the object.
-    if ( color == Color::NONE ) {
+    if ( color == PlayerColor::NONE ) {
         const auto isFlag = [uid]( const auto & part ) { return part._uid == uid && part.icnType == MP2::OBJ_ICN_TYPE_FLAG32; };
         _groundObjectPart.remove_if( isFlag );
         _topObjectPart.remove_if( isFlag );
@@ -1462,7 +1472,19 @@ bool Maps::Tile::removeObjectPartsByUID( const uint32_t objectUID )
         updateObjectType();
 
         setInitialPassability();
+
+        // The remove of the object part may also affect on the passability on the tiles around.
+        // First set the initial passability for tiles.
+        const Indexes tilesAround = getAroundIndexes( _index, 1 );
+        for ( const int32_t tileIndex : tilesAround ) {
+            world.getTile( tileIndex ).setInitialPassability();
+        }
+
+        // Update passability based on initial passability.
         updatePassability();
+        for ( const int32_t tileIndex : tilesAround ) {
+            world.getTile( tileIndex ).updatePassability();
+        }
 
         if ( Heroes::isValidId( _occupantHeroId ) ) {
             Heroes * hero = world.GetHeroes( _occupantHeroId );
@@ -1545,7 +1567,7 @@ void Maps::Tile::updateObjectImageIndex( const uint32_t objectUid, const MP2::Ob
     }
 }
 
-void Maps::Tile::ClearFog( const int colors )
+void Maps::Tile::ClearFog( const PlayerColorsSet colors )
 {
     _fogColors &= ~colors;
 
@@ -1570,6 +1592,19 @@ void Maps::Tile::updateTileObjectIcnIndex( Maps::Tile & tile, const uint32_t uid
 
 void Maps::Tile::updateObjectType()
 {
+    if ( _mainObjectType == MP2::OBJ_EVENT ) {
+        if ( world.GetMapEvent( Maps::GetPoint( _index ) ) == nullptr ) {
+            // No data found for this event type. This may happen in the case of hacked maps.
+            DEBUG_LOG( DBG_AI, DBG_INFO, "Adventure Map event at index " << _index << " is missing!" )
+
+            // Remove the event object type because of the missing data.
+            setMainObjectType( MP2::OBJ_NONE );
+        }
+
+        // Events have no visible parts on the map, so we preserve their type regardless of what part of the object is on the tile.
+        return;
+    }
+
     // After removing an object there could be an object part in the main object part.
     MP2::MapObjectType objectType = getObjectTypeByIcn( _mainObjectPart.icnType, _mainObjectPart.icnIndex );
     if ( MP2::isOffGameActionObject( objectType ) ) {
